@@ -1,8 +1,11 @@
 use std::str::FromStr;
 
 use bio_files_rs;
+use lin_alg::f64::Vec3;
 use pyo3::{exceptions::PyValueError, prelude::*, types::PyType};
 
+mod cif_sf;
+mod gro;
 mod md_params;
 mod mmcif;
 mod mol2;
@@ -68,17 +71,36 @@ impl AtomGeneric {
         self.inner.serial_number
     }
 
+    #[setter(serial_number)]
+    fn serial_number_set(&mut self, v: u32) {
+        self.inner.serial_number = v;
+    }
+
     #[getter]
     fn posit(&self) -> [f64; 3] {
         self.inner.posit.to_arr()
     }
 
+    #[setter(posit)]
+    fn posit_set(&mut self, v: [f64; 3]) -> PyResult<()> {
+        self.inner.posit = Vec3::from_slice(&v)
+            .map_err(|e| pyo3::exceptions::PyValueError::new_err(format!("{e:?}")))?;
+        Ok(())
+    }
+
     #[getter]
     // todo: String for now
     fn element(&self) -> String {
         self.inner.element.to_string()
     }
 
+    #[setter(element)]
+    fn element_set(&mut self, v: String) -> PyResult<()> {
+        self.inner.element = na_seq::Element::from_letter(&v)
+            .map_err(|e| pyo3::exceptions::PyValueError::new_err(e.to_string()))?;
+        Ok(())
+    }
+
     #[getter]
     // todo: String for now
     fn type_in_res(&self) -> Option<String> {
@@ -90,16 +112,31 @@ impl AtomGeneric {
         self.inner.force_field_type.clone()
     }
 
+    #[setter(force_field_type)]
+    fn force_field_type_set(&mut self, v: Option<String>) {
+        self.inner.force_field_type = v;
+    }
+
     #[getter]
     fn partial_charge(&self) -> Option<f32> {
         self.inner.partial_charge
     }
 
+    #[setter(partial_charge)]
+    fn partial_charge_set(&mut self, v: Option<f32>) {
+        self.inner.partial_charge = v;
+    }
+
     #[getter]
     fn hetero(&self) -> bool {
         self.inner.hetero
     }
 
+    #[setter(hetero)]
+    fn hetero_set(&mut self, v: bool) {
+        self.inner.hetero = v;
+    }
+
     fn __repr__(&self) -> String {
         format!("{:?}", self.inner)
     }
@@ -343,7 +380,7 @@ impl ExperimentalMethod {
 }
 
 #[pymodule]
-fn biology_files(py: Python<'_>, m: &Bound<PyModule>) -> PyResult<()> {
+fn biology_files(_py: Python<'_>, m: &Bound<PyModule>) -> PyResult<()> {
     // General
     m.add_class::<AtomGeneric>()?;
     m.add_class::<BondType>()?;
@@ -361,11 +398,19 @@ fn biology_files(py: Python<'_>, m: &Bound<PyModule>) -> PyResult<()> {
     m.add_class::<mmcif::MmCif>()?;
     m.add_class::<mol2::Mol2>()?;
     m.add_class::<sdf::Sdf>()?;
+    m.add_class::<gro::Gro>()?;
     m.add_class::<pdbqt::Pdbqt>()?;
 
+    // Electron density;
+    // todo: Map
+    m.add_class::<cif_sf::CifStructureFactors>()?;
+
     m.add_class::<mol2::MolType>()?;
 
     m.add_class::<md_params::ForceFieldParams>()?;
 
+    m.add_function(wrap_pyfunction!(md_params::load_prmtop, m)?)?;
+    m.add_function(wrap_pyfunction!(md_params::save_prmtop, m)?)?;
+
     Ok(())
 }
