@@ -1,8 +1,15 @@
-use std::path::PathBuf;
+use std::{
+    fs::File,
+    io,
+    path::{Path, PathBuf},
+};
 
 use bio_files_rs;
 use pyo3::{prelude::*, types::PyType};
 
+use crate::{AtomGeneric, BondGeneric};
+
+#[derive(Clone)]
 #[pyclass]
 pub struct ForceFieldParams {
     inner: bio_files_rs::md_params::ForceFieldParams,
@@ -48,3 +55,26 @@ impl ForceFieldParams {
         format!("{:?}", self.inner)
     }
 }
+
+#[pyfunction]
+pub fn save_prmtop(
+    atoms: Vec<PyRef<AtomGeneric>>,
+    params: PyRef<ForceFieldParams>,
+    path: PathBuf,
+) -> io::Result<()> {
+    // Requires that the inner types implement Clone.
+    let atoms: Vec<_> = atoms.into_iter().map(|a| a.inner.clone()).collect();
+    bio_files_rs::prmtop::save_prmtop(&atoms, &params.inner, &path)
+}
+
+#[pyfunction]
+pub fn load_prmtop(path: PathBuf) -> io::Result<(Vec<AtomGeneric>, ForceFieldParams)> {
+    let (atoms, params) = bio_files_rs::prmtop::load_prmtop(&path)?;
+    Ok((
+        atoms
+            .into_iter()
+            .map(|a| AtomGeneric { inner: a })
+            .collect(),
+        ForceFieldParams { inner: params },
+    ))
+}
