@@ -108,10 +108,9 @@ impl MmCif {
         })
     }
 
-    // todo: When implemented in rust.
-    // fn save(&self, path: PathBuf) -> PyResult<()> {
-    //     self.inner.save(&path)
-    // }
+    fn save(&self, path: PathBuf) -> PyResult<()> {
+        Ok(self.inner.save(&path)?)
+    }
 
     #[classmethod]
     fn load(_cls: &Bound<'_, PyType>, path: PathBuf) -> PyResult<Self> {
@@ -120,6 +119,13 @@ impl MmCif {
         })
     }
 
+    #[classmethod]
+    fn load_rcsb(_cls: &Bound<'_, PyType>, ident: &str) -> PyResult<Self> {
+        Ok(Self {
+            inner: bio_files_rs::MmCif::load_rcsb(ident)?,
+        })
+    }
+
     fn __repr__(&self) -> String {
         format!("{:?}", self.inner)
     }
