@@ -16,13 +16,16 @@ use std::{
     str::FromStr,
 };
 
-use na_seq::{AminoAcidGeneral, AtomTypeInRes};
+use lin_alg::f64::Vec3;
+use na_seq::{AminoAcidGeneral, AtomTypeInRes, Element};
 
-/// Data for a MASS entry: e.g. "CT 12.01100" with optional comment
+use crate::{AtomGeneric, BondGeneric, BondType, LipidStandard};
+
+/// Data for a MASS entry: e.g. "CT 12.01100" with optional comment.
 #[derive(Debug, Clone)]
 pub struct MassParams {
     pub atom_type: String,
-    /// AMU
+    /// Atomic mass units (Daltons)
     pub mass: f32,
     // /// ATPOL: Atomic polarizability (Å^3).
     // /// Intended for Slater–Kirkwood or future polarizable models, and unused by Amber (?)
@@ -45,13 +48,6 @@ impl MassParams {
         let atom_type = cols[0].to_string();
         let mass = parse_float(cols[1])?;
 
-        // Skipping polarizability; in for historical reasons?
-        // // Note: This skips comments where this is a missing col[2].
-        // let mut polarizability = 0.0;
-        // if cols.len() >= 3 {
-        //     polarizability = parse_float(cols[2])?;
-        // }
-
         // Note: This skips comments where this is a missing col[2].
         let mut comment = None;
         if cols.len() >= 4 {
@@ -61,7 +57,6 @@ impl MassParams {
         Ok(Self {
             atom_type,
             mass,
-            // polarizability,
             comment,
         })
     }
@@ -122,7 +117,7 @@ pub struct AngleBendingParams {
     pub atom_types: (String, String, String),
     /// Force constant. kcal/mol/rad²
     pub k: f32,
-    /// In degrees.
+    /// In radians.
     pub theta_0: f32,
     pub comment: Option<String>,
 }
@@ -166,9 +161,9 @@ impl AngleBendingParams {
     }
 }
 
-/// Also known as Torsion. Data for both proper, and improper dihedral data.
+/// Also known as Torsion angle.
 ///
-/// Angle between 4 linear covalently-bonded atoms ("dihedral"), or 3 atoms in a hub-and-spoke
+/// Angle between 4 linear covalently-bonded atoms ("proper"), or 3 atoms in a hub-and-spoke
 /// configuration, with atom 3 as the hub ("improper"). In either case, this is the angle between the planes of
 /// atoms 1-2-3, and 2-3-4. (Rotation around the 2-3 bond)
 #[derive(Debug, Clone, Default)]
@@ -178,11 +173,11 @@ pub struct DihedralParams {
     /// Scaling factor used for barrier height.
     /// "Splits the torsion term into individual contributions for
     /// each pair of atoms involved in the torsion."
-    /// Always 1 for improper dihedrdals. (Not present in the Amber files for improper)
+    /// Always 1 for improper dihedrals. (Not present in the Amber files for improper)
     pub divider: u8,
-    /// Also known as V_n., kcal/mol/rad²
+    /// Also known as V_n. kcal/mol.
     pub barrier_height: f32,
-    /// Equilibrium angle, or phase, in radians. Often 0 or τ/2. Maximum (?) energy
+    /// Phase, in radians. Often 0 or τ/2. Maximum energy
     /// is encountered at this value, and other values implied by periodicity.
     /// For example, if this is 0, and periodicity is 3, there is no torsion
     /// force applied for dihedral angles 0, τ/3, and 2τ/3.
@@ -193,7 +188,7 @@ pub struct DihedralParams {
     /// "If the torsion definition has a "negative" periodicity (-2 in the case above), it tells
     /// programs reading the parameter file that additional terms are present for that
     /// particular connectivity.
-    pub periodicity: i8,
+    pub periodicity: u8,
     pub comment: Option<String>,
 }
 
@@ -230,7 +225,10 @@ impl DihedralParams {
 
         let barrier_height_vn = parse_float(cols[col1_i])?;
         let phase = parse_float(cols[col1_i + 1])?.to_radians();
-        let periodicity = parse_float(cols[col1_i + 2])? as i8;
+
+        // A negative periodicity in Amber params indicates that there are additional terms
+        // are present. We ignore those for now.
+        let periodicity = parse_float(cols[col1_i + 2])?.abs() as u8;
 
         // We ignore the remaining cols for now: Source, # of ref geometries used to fit,
         // and RMS deviation of the fit.
@@ -255,16 +253,17 @@ impl DihedralParams {
 }
 
 #[derive(Debug, Clone)]
+/// Represents Lennard Jones parameters. This approximate Pauli Exclusion (i.e. exchange interactions)
+/// with Van Der Waals ones. Note: Amber stores Rmin / 2 in Å. (This is called R star). We convert to σ, which can
+/// be used in more general LJ formulas. The relation: R_min = 2^(1/6) σ. σ = 2 R_star / 2^(1/6)
 /// Amber RM, section 15.1.7
 pub struct LjParams {
     pub atom_type: String,
     /// σ. derived from Van der Waals radius, Å. Note that Amber parameter files use R_min,
-    /// vice σ. This value is σ, which we compute when parsing.
-    ///
-    /// R_min (i, j) = 0.5(R_min_i + R_min_j)
-    /// σ_min (i, j) = 0.5(σ_min_i + σ_min_j)
+    /// vice σ. The value in this field is σ, which we compute when parsing.
     pub sigma: f32,
     /// Energy, kcal/mol. (Represents depth of the potential well).
+    /// σ(i, j) = 0.5 * (σ_i + σ_j)
     /// ε(i, j) = sqrt(ε_i * ε_j)
     pub eps: f32,
 }
@@ -272,7 +271,11 @@ pub struct LjParams {
 impl LjParams {
     /// Parse a single van-der-Waals (Lennard-Jones) parameter line.
     pub fn from_line(line: &str) -> io::Result<Self> {
-        const SIGMA_FACTOR: f32 = 1.122_462_048_309_373; // 2^(1/6)
+        // todo: QC this factor of 2!
+        // 1.122 is 2^(1/6)
+        // todo: We're getting conflicting information on if we should
+        // todo use a factor of 2, or 4 as the prefix here.
+        const SIGMA_FACTOR: f32 = 2. / 1.122_462_048_309_373;
 
         let cols: Vec<_> = line.split_whitespace().collect();
 
@@ -284,10 +287,10 @@ impl LjParams {
         }
 
         let atom_type = cols[0].to_string();
-        let r_min = parse_float(cols[1])?;
+        let r_star = parse_float(cols[1])?;
         let eps = parse_float(cols[2])?;
 
-        let sigma = 2.0 * r_min / SIGMA_FACTOR;
+        let sigma = r_star * SIGMA_FACTOR;
 
         Ok(Self {
             atom_type,
@@ -297,9 +300,9 @@ impl LjParams {
     }
 }
 
-#[derive(Debug)]
+#[derive(Clone, Debug)]
 pub struct ChargeParams {
-    /// The residue-specific ID. We use this value to map forcefield type
+    /// For proteins. The residue-specific ID. We use this value to map forcefield type
     /// to atoms loaded from mmCIF etc; these will have this `type_in_res`, but not
     /// an Amber ff type. We apply the charge here to the atom based on its `type_in_res` and AA type,
     /// and apply its FF type.
@@ -308,36 +311,20 @@ pub struct ChargeParams {
     pub type_in_res: AtomTypeInRes,
     /// "XC", "H1" etc.
     pub ff_type: String,
-    pub charge: f32, // partial charge (q_i)
+    /// Partial charge. Units of elementary charge.
+    pub charge: f32,
 }
 
-// impl ChargeParams {
-//     /// Parse a single van-der-Waals (Lennard-Jones) parameter line.
-//     /// Note: This comes from a .lib file; not .dat or .frcmod. This differs
-//     /// from the other parsings inthis file.
-//     pub fn from_line(line: &str) -> io::Result<Self> {
-//         let cols: Vec<_> = line.split_whitespace().collect();
-//
-//         if cols.len() < 3 {
-//             return Err(io::Error::new(
-//                 ErrorKind::InvalidData,
-//                 "Not enough cols (Charge).",
-//             ));
-//         }
-//
-//         let atom_type = cols[0].to_string();
-//         let r_min = parse_float(cols[1])?;
-//         let eps = parse_float(cols[2])?;
-//
-//         Ok(Self {
-//             atom_type,
-//             atom_type,
-//             charge
-//         })
-//     }
-// }
+/// See notes on `ChargeParams`; equivalent here.
+#[derive(Clone, Debug)]
+pub struct ChargeParamsLipid {
+    // pub type_in_res: AtomTypeInLipid,
+    pub type_in_res: String,
+    pub ff_type: String,
+    pub charge: f32,
+}
 
-/// Top-level lib, dat or frcmod data. We store the name-tuples in fields, vice as HashMaps here,
+/// Top-level lib, dat, or frcmod data. We store the name-tuples in fields, vice as HashMaps here,
 /// for parsing flexibility.
 ///
 /// Note that we don't include partial charges here, as they come from Mol2 files; this struct
@@ -375,12 +362,15 @@ pub struct ForceFieldParams {
     pub angle: HashMap<(String, String, String), AngleBendingParams>,
     /// Angle between 4 linear covalently-bonded atoms (3 bonds). This is
     /// the angle between the planes of atoms 1-2-3, and 2-3-4. (Rotation around the 2-3 bond)
-    pub dihedral: HashMap<(String, String, String, String), DihedralParams>,
+    /// This is a Vec, as there can be multiple terms for proper dihedrals. (Negative
+    /// periodicity is a flag meaning there are follow-on terms)
+    pub dihedral: HashMap<(String, String, String, String), Vec<DihedralParams>>,
     /// Angle between 4 covalently-bonded atoms (3 bonds), in a hub-and-spoke
     /// arrangement. The third atom is the hub. This is the angle between the planes of
     /// atoms 1-2-3, and 2-3-4. Note that these are generally only included for planar configurations,
     /// and always hold a planar dihedral shape. (e.g. τ/2 with symmetry 2)
-    pub improper: HashMap<(String, String, String, String), DihedralParams>,
+    /// It's possible, but unlikely there can be more than one improper term
+    pub improper: HashMap<(String, String, String, String), Vec<DihedralParams>>,
     pub mass: HashMap<String, MassParams>,
     pub lennard_jones: HashMap<String, LjParams>,
 }
@@ -403,12 +393,21 @@ impl ForceFieldParams {
             result.angle.insert(val.atom_types.clone(), val.clone());
         }
 
+        // Insert, or append, as required. There can be multiple proper dihedral terms.
         for val in &params.dihedral {
-            result.dihedral.insert(val.atom_types.clone(), val.clone());
+            result
+                .dihedral
+                .entry(val.atom_types.clone())
+                .and_modify(|v| v.push(val.clone()))
+                .or_insert_with(|| vec![val.clone()]);
         }
 
         for val in &params.improper {
-            result.improper.insert(val.atom_types.clone(), val.clone());
+            result
+                .improper
+                .entry(val.atom_types.clone())
+                .and_modify(|v| v.push(val.clone()))
+                .or_insert_with(|| vec![val.clone()]);
         }
 
         for val in &params.lennard_jones {
@@ -440,46 +439,109 @@ impl ForceFieldParams {
         Ok(Self::new(&ForceFieldParamsVec::load_dat(path)?))
     }
 
+    /// For the `get_` methods below. Expand possible wildcard forms of an atom type, keeping priority order:
+    /// 1. Exact atom name
+    /// 2. Pattern with same first letter and '*'
+    /// 3. Global wildcard "X"
+    fn wildcard_variants(atom: &str) -> Vec<String> {
+        let mut out = Vec::new();
+        out.push(atom.to_string()); // exact
+
+        if atom.len() > 0 {
+            let first = atom.chars().next().unwrap();
+            // Only add meaningful ones like C*, N*, O*, etc.
+            if first.is_ascii_alphabetic() {
+                out.push(format!("{}*", first));
+            }
+        }
+        out.push("X".to_string());
+        out
+    }
+
     /// A utility function that handles proper and improper dihedral data,
     /// tries both atom orders, and falls back to wildcard (“X”) matches on
     /// the outer atoms when an exact hit is not found.
+    pub fn get_bond(&self, atom_types: &(String, String)) -> Option<&BondStretchingParams> {
+        let a_variants = Self::wildcard_variants(&atom_types.0);
+        let b_variants = Self::wildcard_variants(&atom_types.1);
+
+        // Priority: exact before partial before X
+        for a in &a_variants {
+            for b in &b_variants {
+                // try both orders
+                for &(k0, k1) in &[(a, b), (b, a)] {
+                    let key = (k0.clone(), k1.clone());
+                    if let Some(hit) = self.bond.get(&key) {
+                        return Some(hit);
+                    }
+                }
+            }
+        }
+        None
+    }
+
+    // todo: YOu may need to augment all these helps with support for "C*", "N*" etc.
+
+    /// A utility function that handles proper and improper dihedral data,
+    /// tries both atom orders, and falls back to wildcard (“X”) matches on
+    /// the outer atoms when an exact hit is not found.
+    pub fn get_valence_angle(
+        &self,
+        atom_types: &(String, String, String),
+    ) -> Option<&AngleBendingParams> {
+        let a_variants = Self::wildcard_variants(&atom_types.0);
+        let b_variants = Self::wildcard_variants(&atom_types.1);
+        let c_variants = Self::wildcard_variants(&atom_types.2);
+
+        // Try combinations in both directions (a-b-c and c-b-a)
+        for a in &a_variants {
+            for b in &b_variants {
+                for c in &c_variants {
+                    for &(k0, k1, k2) in &[(a, b, c), (c, b, a)] {
+                        let key = (k0.clone(), k1.clone(), k2.clone());
+                        if let Some(hit) = self.angle.get(&key) {
+                            return Some(hit);
+                        }
+                    }
+                }
+            }
+        }
+        None
+    }
+
+    /// A utility function that handles proper and improper dihedral data,
+    /// tries both atom orders, and falls back to wildcard (“X”) matches on
+    /// the outer atoms when an exact hit is not found.
+    ///
+    /// We return multiple, as there can be multiple dihedrals for a single atom type set;
+    /// we add them during computations.
     pub fn get_dihedral(
         &self,
         atom_types: &(String, String, String, String),
-        proper: bool, // todo: Experimenting.
-    ) -> Option<&DihedralParams> {
-        let a = atom_types.0.as_str();
-        let b = atom_types.1.as_str();
-        let c = atom_types.2.as_str();
-        let d = atom_types.3.as_str();
+        proper: bool,
+    ) -> Option<&Vec<DihedralParams>> {
+        let a_variants = Self::wildcard_variants(&atom_types.0);
+        let b_variants = Self::wildcard_variants(&atom_types.1);
+        let c_variants = Self::wildcard_variants(&atom_types.2);
+        let d_variants = Self::wildcard_variants(&atom_types.3);
 
-        const X: &str = "X";
-        let candidates = [
-            // Exact
-            (a, b, c, d),
-            (d, c, b, a),
-            // X on one side
-            (X, b, c, d),
-            (X, c, b, a),
-            (a, b, c, X),
-            (d, c, b, X),
-            // Xs on both sides.
-            (X, b, c, X),
-            (X, c, b, X),
-        ];
-
-        for &(k0, k1, k2, k3) in &candidates {
-            // Build a temporary `String` tuple only for the actual lookup
-            let key = (k0.to_owned(), k1.to_owned(), k2.to_owned(), k3.to_owned());
-
-            let hit = if proper {
-                self.dihedral.get(&key)
-            } else {
-                self.improper.get(&key)
-            };
-
-            if hit.is_some() {
-                return hit;
+        for a in &a_variants {
+            for b in &b_variants {
+                for c in &c_variants {
+                    for d in &d_variants {
+                        for &(k0, k1, k2, k3) in &[(a, b, c, d), (d, c, b, a)] {
+                            let key = (k0.clone(), k1.clone(), k2.clone(), k3.clone());
+                            let hit = if proper {
+                                self.dihedral.get(&key)
+                            } else {
+                                self.improper.get(&key)
+                            };
+                            if let Some(h) = hit {
+                                return Some(h);
+                            }
+                        }
+                    }
+                }
             }
         }
         None
@@ -599,6 +661,89 @@ pub fn parse_amino_charges(text: &str) -> io::Result<HashMap<AminoAcidGeneral, V
     Ok(result)
 }
 
+// todo: This is DRY with the parse_amino_charges fn above. Fix it. Too much repetition for too little diff.
+pub fn parse_lipid_charges(
+    text: &str,
+) -> io::Result<HashMap<LipidStandard, Vec<ChargeParamsLipid>>> {
+    enum Mode {
+        Scan,                           // not inside an atoms table
+        InAtoms { res: LipidStandard }, // currently reading atom lines for this residue
+    }
+
+    let mut state = Mode::Scan;
+    let mut result: HashMap<LipidStandard, Vec<ChargeParamsLipid>> = HashMap::new();
+
+    let lines: Vec<&str> = text.lines().collect();
+
+    for line in lines {
+        let ltrim = line.trim_start();
+
+        // Section headers
+        if let Some(rest) = ltrim.strip_prefix("!entry.") {
+            state = Mode::Scan;
+
+            if let Some((tag, tail)) = rest.split_once('.') {
+                // We only care about "<RES>.unit.atoms table"
+                if tail.starts_with("unit.atoms table") {
+                    let Ok(aa) = LipidStandard::from_str(tag) else {
+                        return Err(io::Error::new(
+                            ErrorKind::InvalidData,
+                            format!("Unable to parse lipid from lib: {tag}"),
+                        ));
+                    };
+
+                    state = Mode::InAtoms { res: aa };
+
+                    result.entry(aa).or_default(); // make sure map key exists
+                }
+            }
+            continue;
+        }
+
+        // If inside atoms table, parse data line
+        if let Mode::InAtoms { ref res } = state {
+            // tables end when we hit an empty line or a comment
+            if ltrim.is_empty() || ltrim.starts_with('!') {
+                state = Mode::Scan;
+                continue;
+            }
+
+            let mut tokens = Vec::<&str>::new();
+            let mut in_quote = false;
+            let mut start = 0usize;
+            let bytes = ltrim.as_bytes();
+            for (i, &b) in bytes.iter().enumerate() {
+                match b {
+                    b'"' => in_quote = !in_quote,
+                    b' ' | b'\t' if !in_quote => {
+                        if start < i {
+                            tokens.push(&ltrim[start..i]);
+                        }
+                        start = i + 1;
+                    }
+                    _ => {}
+                }
+            }
+            if start < ltrim.len() {
+                tokens.push(&ltrim[start..]);
+            }
+
+            let type_in_res = tokens[0].trim_matches('"').to_string();
+            let ff_type = tokens[1].trim_matches('"').to_string();
+            let charge = parse_float(tokens.last().unwrap())?;
+
+            result.get_mut(res).unwrap().push(ChargeParamsLipid {
+                // type_in_res: AtomTypeInLipid::from_str(&type_in_res)?,
+                type_in_res,
+                ff_type,
+                charge,
+            });
+        }
+    }
+
+    Ok(result)
+}
+
 pub fn load_amino_charges(path: &Path) -> io::Result<HashMap<AminoAcidGeneral, Vec<ChargeParams>>> {
     let mut file = File::open(path)?;
     let mut buffer = Vec::new();
@@ -609,3 +754,218 @@ pub fn load_amino_charges(path: &Path) -> io::Result<HashMap<AminoAcidGeneral, V
 
     parse_amino_charges(&data_str)
 }
+
+// todo: Consider an amino acid variant of this too!
+/// Creates a set of lipid atom and bonds for all items in a `.lib` file, e.g. `lipid21.lib` from Amber.
+/// The hashmap key is the lipid name, e.g. "AR", "CHL" etc.
+pub fn load_lipid_templates(
+    text: &str,
+) -> io::Result<HashMap<String, (Vec<AtomGeneric>, Vec<BondGeneric>)>> {
+    #[derive(Clone)]
+    struct AtomRow {
+        name: String,    // per-residue atom name (eg. "C116")
+        ff_type: String, // Amber/GAFF atom type (eg. "cD")
+        z: u32,          // atomic number
+        q: f64,          // partial charge
+    }
+
+    #[derive(Default)]
+    struct Work {
+        atoms: Vec<AtomRow>,
+        positions: Vec<Vec3>,
+        bonds: Vec<(u32, u32, u32)>, // (a1, a2, flag)
+    }
+
+    enum Sec {
+        Atoms,
+        Positions,
+        Connectivity,
+    }
+
+    let mut out: HashMap<String, (Vec<AtomGeneric>, Vec<BondGeneric>)> = HashMap::new();
+    let mut cur_key: Option<String> = None;
+    let mut cur_sec: Option<Sec> = None;
+    let mut cur: Work = Work::default();
+
+    let element_from_z = |z: u32| -> Element {
+        match z {
+            1 => Element::Hydrogen,
+            6 => Element::Carbon,
+            7 => Element::Nitrogen,
+            8 => Element::Oxygen,
+            9 => Element::Fluorine,
+            15 => Element::Phosphorus,
+            16 => Element::Sulfur,
+            17 => Element::Chlorine,
+            35 => Element::Bromine,
+            53 => Element::Iodine,
+            _ => Element::Tellurium, // fallback; adjust if you have a dedicated Unknown
+        }
+    };
+
+    let bond_from_flag = |f: u32| -> BondType {
+        match f {
+            1 => BondType::Single,
+            2 => BondType::Double,
+            3 => BondType::Triple,
+            // Add/adjust if you encode aromatic or order via other bits
+            _ => BondType::Single,
+        }
+    };
+
+    let mut finalize = |key: Option<String>, work: &mut Work| {
+        if let Some(k) = key {
+            if !work.atoms.is_empty() {
+                let n = work.atoms.len();
+
+                // Ensure positions length
+                if work.positions.len() < n {
+                    work.positions.extend(
+                        std::iter::repeat_with(|| Vec3::new(0.0, 0.0, 0.0))
+                            .take(n - work.positions.len()),
+                    );
+                }
+
+                // Build atoms
+                let atoms: Vec<AtomGeneric> = work
+                    .atoms
+                    .iter()
+                    .enumerate()
+                    .map(|(i, ar)| AtomGeneric {
+                        serial_number: (i as u32) + 1,
+                        posit: work.positions[i],
+                        element: element_from_z(ar.z),
+                        type_in_res: None,
+                        type_in_res_lipid: Some(ar.name.clone()),
+                        force_field_type: Some(ar.ff_type.clone()),
+                        partial_charge: Some(ar.q as f32),
+                        hetero: false,
+                        occupancy: None,
+                        alt_conformation_id: None,
+                    })
+                    .collect();
+
+                // Build bonds
+                let bonds: Vec<BondGeneric> = work
+                    .bonds
+                    .iter()
+                    .map(|&(a1, a2, fl)| BondGeneric {
+                        bond_type: bond_from_flag(fl),
+                        atom_0_sn: a1,
+                        atom_1_sn: a2,
+                    })
+                    .collect();
+
+                out.insert(k, (atoms, bonds));
+            }
+            *work = Work::default();
+        }
+    };
+
+    for line in text.lines() {
+        let l = line.trim_start();
+
+        if let Some(rest) = l.strip_prefix("!entry.") {
+            // New entry section header
+            // Forms we care about:
+            // !entry.<NAME>.unit.atoms table ...
+            // !entry.<NAME>.unit.positions table ...
+            // !entry.<NAME>.unit.connectivity table ...
+            // Extract key = between "!entry." and ".unit."
+            if let Some(dot_unit_idx) = rest.find(".unit.") {
+                let key = &rest[..dot_unit_idx];
+                let after_unit = &rest[dot_unit_idx + ".unit.".len()..];
+
+                // If switching to a new NAME (key) and we already had a current key, finalize the previous lipid
+                let key_changed = match &cur_key {
+                    Some(k) => k != key,
+                    None => true,
+                };
+                if key_changed {
+                    finalize(cur_key.take(), &mut cur);
+                    cur_key = Some(key.to_string());
+                }
+
+                if after_unit.starts_with("atoms table") {
+                    cur_sec = Some(Sec::Atoms);
+                    continue;
+                } else if after_unit.starts_with("positions table") {
+                    cur_sec = Some(Sec::Positions);
+                    continue;
+                } else if after_unit.starts_with("connectivity table") {
+                    cur_sec = Some(Sec::Connectivity);
+                    continue;
+                } else {
+                    // Section we don't parse; leave state None so we skip rows
+                    cur_sec = None;
+                    continue;
+                }
+            } else {
+                // Some other entry; ignore
+                cur_sec = None;
+                continue;
+            }
+        }
+
+        match cur_sec {
+            Some(Sec::Atoms) => {
+                // Expect:  "NAME" "TYPE" 0 1 131073 1 6 0.039100
+                // Grab the two quoted fields first.
+                let bytes = l.as_bytes();
+                let mut qpos = Vec::with_capacity(4);
+                for (i, &b) in bytes.iter().enumerate() {
+                    if b == b'"' {
+                        qpos.push(i);
+                    }
+                    if qpos.len() == 4 {
+                        break;
+                    }
+                }
+                if qpos.len() == 4 {
+                    let name = &l[qpos[0] + 1..qpos[1]];
+                    let ff_type = &l[qpos[2] + 1..qpos[3]];
+                    let tail = &l[qpos[3] + 1..];
+                    // Remaining numeric tokens; we need elmnt (5th) and chg (6th) after the two quoted tokens
+                    // Layout: typex resx flags seq elmnt chg
+                    let nums: Vec<&str> = tail.split_whitespace().collect();
+                    if nums.len() >= 6 {
+                        let elmnt = nums[4].parse::<u32>().unwrap_or(0);
+                        let chg = nums[5].parse::<f64>().unwrap_or(0.0);
+                        cur.atoms.push(AtomRow {
+                            name: name.to_string(),
+                            ff_type: ff_type.to_string(),
+                            z: elmnt,
+                            q: chg,
+                        });
+                    }
+                }
+            }
+            Some(Sec::Positions) => {
+                // Expect three floats per line
+                let mut it = l.split_whitespace();
+                if let (Some(x), Some(y), Some(z)) = (it.next(), it.next(), it.next())
+                    && let (Ok(xv), Ok(yv), Ok(zv)) =
+                        (x.parse::<f64>(), y.parse::<f64>(), z.parse::<f64>())
+                {
+                    cur.positions.push(Vec3::new(xv, yv, zv));
+                }
+            }
+            Some(Sec::Connectivity) => {
+                // Expect three ints: atom1x atom2x flags
+                let mut it = l.split_whitespace();
+                if let (Some(a1), Some(a2), Some(flg)) = (it.next(), it.next(), it.next())
+                    && let (Ok(a1v), Ok(a2v), Ok(fv)) =
+                        (a1.parse::<u32>(), a2.parse::<u32>(), flg.parse::<u32>())
+                {
+                    cur.bonds.push((a1v, a2v, fv));
+                }
+            }
+            None => {}
+        }
+    }
+
+    // Finalize last lipid
+    finalize(cur_key.take(), &mut cur);
+
+    Ok(out)
+}
