@@ -1,17 +1,18 @@
 //! For opening Mol2 files. These are common molecular descriptions for ligands.
-//! [This unofficial resource](https://chemicbook.com/2021/02/20/mol2-file-format-explained-for-beginners-part-2.html)
-//! descripts the format.
+//! We implement the [Tripos Mol2 spec](https://zhanggroup.org/DockRMSD/mol2.pdf) variant,
+//! as it's used by Amber Geostd, and has a published spec.
 
 use std::{
     collections::HashMap,
-    fmt,
+    fmt, fs,
     fs::File,
     io,
-    io::{ErrorKind, Read, Write},
+    io::{ErrorKind, Write},
     path::Path,
     str::FromStr,
 };
 
+use bio_apis::amber_geostd;
 use lin_alg::f64::Vec3;
 use na_seq::{AtomTypeInRes, Element};
 
@@ -120,6 +121,10 @@ impl FromStr for ChargeType {
     }
 }
 
+/// This implements the [Tripos Mol2 spec](https://zhanggroup.org/DockRMSD/mol2.pdf).
+/// It's a format used for small organic molecules, and may include force field names and partial
+/// charges, making it suitable for molecular dynamics applications. This struct will likely
+/// be used as an intermediate format, and converted to something application-specific.
 // todo: Combine this and SDF into one struct?
 #[derive(Clone, Debug)]
 pub struct Mol2 {
@@ -238,18 +243,33 @@ impl Mol2 {
                     atom_name = before_dot.to_string();
                 }
 
-                let element = match Element::from_letter(&atom_name) {
+                let mut element = match Element::from_letter(
+                    &atom_name
+                        .chars()
+                        .filter(|c| c.is_ascii_alphabetic())
+                        .collect::<String>(),
+                ) {
                     Ok(l) => l,
                     Err(e) => {
                         if atom_name.len() > 1 {
-                            // It might be something like "c3", "c1" etc."
-                            Element::from_letter(&atom_name[0..1])?
+                            // Try this:
+                            if atom_name.starts_with("BR") {
+                                Element::Bromine
+                            } else {
+                                // It might be something like "c3", "c1" etc.; try the first letter only.
+                                Element::from_letter(&atom_name[0..1])?
+                            }
                         } else {
                             return Err(e);
                         }
                     }
                 };
 
+                // A manual override. Not Calcium.
+                if atom_name == "CA" {
+                    element = Element::Carbon;
+                }
+
                 let x = cols[2].parse::<f64>().map_err(|_| {
                     io::Error::new(ErrorKind::InvalidData, "Could not parse X coordinate")
                 })?;
@@ -284,10 +304,10 @@ impl Mol2 {
                     type_in_res,
                     posit: Vec3 { x, y, z }, // or however you store coordinates
                     element,
-                    occupancy: None,
                     partial_charge,
                     force_field_type: Some(cols[5].to_string()),
                     hetero: true,
+                    ..Default::default()
                 });
             }
 
@@ -426,7 +446,7 @@ impl Mol2 {
                 i + 1,
                 bond.atom_0_sn,
                 bond.atom_1_sn,
-                bond.bond_type.to_string(),
+                bond.bond_type.to_mol2_str(),
             )?;
         }
 
@@ -434,13 +454,14 @@ impl Mol2 {
     }
 
     pub fn load(path: &Path) -> io::Result<Self> {
-        let mut file = File::open(path)?;
-        let mut buffer = Vec::new();
-        file.read_to_end(&mut buffer)?;
-
-        let data_str: String = String::from_utf8(buffer)
-            .map_err(|_| io::Error::new(ErrorKind::InvalidData, "Invalid UTF8"))?;
+        let data_str = fs::read_to_string(path)?;
+        Self::new(&data_str)
+    }
 
+    /// Download  rom our Amber Geostd DB using a PDBe/Amber ID.
+    pub fn load_amber_geostd(ident: &str) -> io::Result<Self> {
+        let data_str = amber_geostd::load_mol2(ident)
+            .map_err(|e| io::Error::new(ErrorKind::Other, format!("Error loading: {e:?}")))?;
         Self::new(&data_str)
     }
 }
