@@ -3,9 +3,10 @@
 
 use std::{
     fmt::Display,
+    fs,
     fs::File,
     io,
-    io::{ErrorKind, Read, Write},
+    io::{ErrorKind, Write},
     path::Path,
     str::FromStr,
 };
@@ -190,15 +191,10 @@ impl Pdbqt {
                     posit: Vec3 { x, y, z },
                     element,
                     type_in_res: Some(type_in_res),
-                    // role,
-                    // residue: None,
-                    // chain: None,
                     hetero,
                     occupancy,
-                    // temperature_factor,
                     partial_charge,
-                    force_field_type: None,
-                    // dock_type,
+                    ..Default::default()
                 });
 
                 // atoms.push(Atom {
@@ -306,7 +302,7 @@ impl Pdbqt {
             // - columns 71..76: partial charge
             // - columns 77..78: autodock type
 
-            let mut res_num = 1;
+            let res_num = 1;
 
             // let residue_name = if ligand.is_some() {
             //     "UNL".to_owned()
@@ -381,13 +377,7 @@ impl Pdbqt {
     }
 
     pub fn load(path: &Path) -> io::Result<Self> {
-        let mut file = File::open(path)?;
-        let mut buffer = Vec::new();
-        file.read_to_end(&mut buffer)?;
-
-        let data_str: String = String::from_utf8(buffer)
-            .map_err(|_| io::Error::new(ErrorKind::InvalidData, "Invalid UTF8"))?;
-
+        let data_str = fs::read_to_string(path)?;
         Self::new(&data_str)
     }
 }
