@@ -3,13 +3,15 @@
 
 use std::{
     collections::HashMap,
+    fs,
     fs::File,
     io,
-    io::{ErrorKind, Read, Write},
+    io::{ErrorKind, Write},
     path::Path,
     str::FromStr,
 };
 
+use bio_apis::{drugbank, pdbe, pubchem};
 use lin_alg::f64::Vec3;
 use na_seq::Element;
 
@@ -17,7 +19,9 @@ use crate::{
     AtomGeneric, BondGeneric, BondType, ChainGeneric, Mol2, ResidueEnd, ResidueGeneric, ResidueType,
 };
 
-// todo: Combine this and Mol2 into one struct?
+/// It's a format used for small organic molecules, and is a common format on online databases
+/// like PubChem and Drugbank. This struct will likely
+/// be used as an intermediate format, and converted to something application-specific.
 #[derive(Clone, Debug)]
 pub struct Sdf {
     pub ident: String,
@@ -123,13 +127,10 @@ impl Sdf {
             atoms.push(AtomGeneric {
                 // SDF doesn't explicitly include incices.
                 serial_number: (i - first_atom_line) as u32 + 1,
-                type_in_res: None,
                 posit: Vec3 { x, y, z }, // or however you store coordinates
                 element: Element::from_letter(element)?,
-                occupancy: None,
-                partial_charge: None,
-                force_field_type: None,
                 hetero: true,
+                ..Default::default()
             });
         }
 
@@ -175,12 +176,12 @@ impl Sdf {
                     pubchem_cid = Some(v);
                 }
             }
-            if line.contains("> <DATABASE_ID>") {
-                if let Some(value_line) = lines.get(i + 1) {
-                    let value = value_line.trim();
-                    if !value.is_empty() {
-                        drugbank_id = Some(value.to_string());
-                    }
+            if line.contains("> <DATABASE_ID>")
+                && let Some(value_line) = lines.get(i + 1)
+            {
+                let value = value_line.trim();
+                if !value.is_empty() {
+                    drugbank_id = Some(value.to_string());
                 }
             }
         }
@@ -234,71 +235,67 @@ impl Sdf {
                 if line == "$$$$" {
                     break;
                 }
-                if line.starts_with('>') {
-                    if let (Some(l), Some(r)) = (line.find('<'), line.rfind('>')) {
-                        if r > l + 1 {
-                            let key = &line[l + 1..r];
-                            idx += 1;
+                if line.starts_with('>')
+                    && let (Some(l), Some(r)) = (line.find('<'), line.rfind('>'))
+                    && r > l + 1
+                {
+                    let key = &line[l + 1..r];
+                    idx += 1;
 
-                            let mut vals: Vec<&str> = Vec::new();
-                            while idx < lines.len() {
-                                let v = lines[idx];
-                                let v_trim = v.trim_end();
-                                if v_trim.is_empty()
-                                    || v_trim == "$$$$"
-                                    || v_trim.starts_with("> <")
-                                {
-                                    break;
-                                }
-                                vals.push(v_trim);
-                                idx += 1;
-                            }
-                            md.insert(key.to_string(), vals.join("\n"));
+                    let mut vals: Vec<&str> = Vec::new();
+                    while idx < lines.len() {
+                        let v = lines[idx];
+                        let v_trim = v.trim_end();
+                        if v_trim.is_empty() || v_trim == "$$$$" || v_trim.starts_with("> <") {
+                            break;
+                        }
+                        vals.push(v_trim);
+                        idx += 1;
+                    }
+                    md.insert(key.to_string(), vals.join("\n"));
 
-                            // OpenFF format.
-                            if key == "atom.dprop.PartialCharge" {
-                                let charges: Vec<_> = lines[idx + 1].split_whitespace().collect();
-                                for (i, q) in charges.into_iter().enumerate() {
-                                    atoms[i].partial_charge = Some(q.parse().unwrap_or(0.));
-                                }
-                            }
-
-                            // Pubchem format.
-                            if key == "PUBCHEM_MMFF94_PARTIAL_CHARGES" {
-                                if vals.is_empty() {
-                                    eprintln!("No values for PUBCHEM_MMFF94_PARTIAL_CHARGES");
-                                } else {
-                                    let n = vals[0].trim().parse::<usize>().unwrap_or(0);
-                                    if vals.len().saturating_sub(1) != n {
-                                        eprintln!(
-                                            "Charge count mismatch: expected {}, got {}",
-                                            n,
-                                            vals.len().saturating_sub(1)
-                                        );
-                                    }
-                                    for line in vals.iter().skip(1).take(n) {
-                                        let mut it = line.split_whitespace();
-                                        let i1 = it.next().and_then(|s| s.parse::<usize>().ok());
-                                        let q = it.next().and_then(|s| s.parse::<f32>().ok());
-
-                                        if let (Some(i1), Some(q)) = (i1, q) {
-                                            if (1..=atoms.len()).contains(&i1) {
-                                                atoms[i1 - 1].partial_charge = Some(q); // 1-based -> 0-based
-                                            } else {
-                                                eprintln!(
-                                                    "Atom index {} out of range (n_atoms={})",
-                                                    i1,
-                                                    atoms.len()
-                                                );
-                                            }
-                                        }
-                                    }
-                                }
-                            }
-
-                            continue;
+                    // OpenFF format.
+                    if key == "atom.dprop.PartialCharge" {
+                        let charges: Vec<_> = lines[idx + 1].split_whitespace().collect();
+                        for (i, q) in charges.into_iter().enumerate() {
+                            atoms[i].partial_charge = Some(q.parse().unwrap_or(0.));
                         }
                     }
+
+                    // Pubchem format.
+                    if key == "PUBCHEM_MMFF94_PARTIAL_CHARGES" {
+                        if vals.is_empty() {
+                            eprintln!("No values for PUBCHEM_MMFF94_PARTIAL_CHARGES");
+                        } else {
+                            let n = vals[0].trim().parse::<usize>().unwrap_or(0);
+                            if vals.len().saturating_sub(1) != n {
+                                eprintln!(
+                                    "Charge count mismatch: expected {}, got {}",
+                                    n,
+                                    vals.len().saturating_sub(1)
+                                );
+                            }
+                            for line in vals.iter().skip(1).take(n) {
+                                let mut it = line.split_whitespace();
+                                let i1 = it.next().and_then(|s| s.parse::<usize>().ok());
+                                let q = it.next().and_then(|s| s.parse::<f32>().ok());
+
+                                if let (Some(i1), Some(q)) = (i1, q) {
+                                    if (1..=atoms.len()).contains(&i1) {
+                                        atoms[i1 - 1].partial_charge = Some(q); // 1-based -> 0-based
+                                    } else {
+                                        eprintln!(
+                                            "Atom index {} out of range (n_atoms={})",
+                                            i1,
+                                            atoms.len()
+                                        );
+                                    }
+                                }
+                            }
+                        }
+                    }
+
+                    continue;
                 }
                 idx += 1;
             }
@@ -419,13 +416,28 @@ impl Sdf {
 
     // todo: Generic fn for this and save, among all text-based types.
     pub fn load(path: &Path) -> io::Result<Self> {
-        let mut file = File::open(path)?;
-        let mut buffer = Vec::new();
-        file.read_to_end(&mut buffer)?;
+        let data_str = fs::read_to_string(path)?;
+        Self::new(&data_str)
+    }
 
-        let data_str: String = String::from_utf8(buffer)
-            .map_err(|_| io::Error::new(ErrorKind::InvalidData, "Invalid UTF8"))?;
+    /// Download from DrugBank from a Drugbank ID.
+    pub fn load_drugbank(ident: &str) -> io::Result<Self> {
+        let data_str = drugbank::load_sdf(ident)
+            .map_err(|e| io::Error::new(ErrorKind::Other, format!("Error loading: {e:?}")))?;
+        Self::new(&data_str)
+    }
 
+    /// Download from PubChem from a CID.
+    pub fn load_pubchem(cid: u32) -> io::Result<Self> {
+        let data_str = pubchem::load_sdf(cid)
+            .map_err(|e| io::Error::new(ErrorKind::Other, format!("Error loading: {e:?}")))?;
+        Self::new(&data_str)
+    }
+
+    /// Download from PDBe from a PDBe ID.
+    pub fn load_pdbe(ident: &str) -> io::Result<Self> {
+        let data_str = pdbe::load_sdf(ident)
+            .map_err(|e| io::Error::new(ErrorKind::Other, format!("Error loading: {e:?}")))?;
         Self::new(&data_str)
     }
 }
